#include "U8glib.h"
#include <arduino.h>

/**********************************************************************
 DISPLAY
 **********************************************************************
 PIN Konfig
 13 SCK
 12 MISO
 11 MOSI /DIN
 10 SS /CS
 9  DC /A0
 8  RESET
 7  BackLight
 ***********************************************************************/

//U8GLIB_PCD8544 u8g(13, 11, 10,9, 8);// SCK, MOSI, CS, A0,RESET .... SW SPI
U8GLIB_PCD8544 u8g(10, 9, 8); // CS, A0, RESET ..... HW SPI
#define BACKLIGHT 7
#define FONTHEIGTH 7

/***********************************************************************
 *SERVOs steering, speed
 ***********************************************************************
 * auf folgenden Arduino Pins befinden sich die Servoanschlsse:
 *
 *  5 ... Platinenbeschriftung TH
 *  6 ... Platinenbeschriftung ST
 *  ********************************************************************/

#include <Servo.h>

Servo steeringServo;  // SteeringServo
Servo speedServo;  // SteeringServo

/*************************************************************************
 * IR-Length Sensors
 *************************************************************************
 * auf folgenden PINs befinden sich die IR Sensoren
 *
 * A0 ... Platinenbeschriftung CO12
 * A1 ... Platinenbeschriftung CO13
 * A2 ... Platinenbeschriftung CO14
 * A3 ... VBat
 *
 * Anmerkung: am AREF Eingang des ADCs liegt eine Referenzspannung von 2.048V
 *************************************************************************/

#define LEFTSENSOR A0
#define MIDDLESENSOR A1
#define RIGTHSENSOR A2
#define VBAT A3

/*************************************************************************/

/*************************************************************************
 * Start Stop Tasten
 *************************************************************************
 * auf folgenden PINs befinden sich die Start und die Stop Taste
 *
 * 3 ... Stop
 * 4 ... Start
 *
 * Anmerkung: die Tasten verwenden negative logik
 *************************************************************************/
#define STARTBUTTON 4 //Platinenbeschriftung Start
#define STOPBUTTON 3  //Platinenbeschriftung Stop

/**************************************************************************
 * Global Variablen
 **************************************************************************/
unsigned char runMode = 0; //Variable = 0 ... stop

//The setup function is called once at startup of the sketch
void setup() {
	//attach Servo objects to the corresponding PINs
	steeringServo.attach(5);
	steeringServo.write(90); //Set it to neutral position
	speedServo.attach(6);
	speedServo.write(90); //Set it to neutral position

	//Hintergrundbeleuchungs pin konfigurieren
	pinMode(BACKLIGHT, OUTPUT);
}

// The loop function is called in an endless loop
void loop() {

	//First get all data
	int leftDistance = analogRead(LEFTSENSOR);
	int middleDistance = analogRead(MIDDLESENSOR);
	int rightDistance = analogRead(RIGTHSENSOR);
	int vBat = analogRead(VBAT);


	//Now check the Button and run control strategies

	if (digitalRead(STOPBUTTON) == LOW)
		runMode = 0; //stopen des Fahrzeugs
	else {
		if (digitalRead(STARTBUTTON) == LOW)
			runMode = 1; //fahrzeug darf fahren
	}

	if (runMode == 1) {
		//Geschwindigkeit
		speedServo.write(110); //lowspeed ahead

		//Lenkung (Steering)
		int lenkung = ((long) (leftDistance - middleDistance)) * 90 / 1024;
		steeringServo.write(lenkung + 90);
	}

	//finally update display

	u8g.setFont(u8g_font_4x6);
	u8g.firstPage();

	do {
		draw(leftDistance, middleDistance, rightDistance, vBat);
	} while (u8g.nextPage());

}

void displaySensorValue(const int value, const char *S, const int xPos,
		const int yPos) {

	u8g.setPrintPos(xPos, yPos);
	u8g.print(S);
	u8g.print(value);

	//calc box
	int boxWidth = ((long) value) * 42 / 1024;

	u8g.drawFrame(84 - boxWidth, yPos - FONTHEIGTH, boxWidth, FONTHEIGTH - 1);
}

void draw(const int leftDistance, const int middleDistance,
		const int rightDistance, const int vBat) {
	//Header
	u8g.setFont(u8g_font_4x6);
	u8g.drawStr(0, 7, "Crazy Car");

	displaySensorValue(leftDistance, "left ", 0, FONTHEIGTH * 3);
	displaySensorValue(middleDistance, "middle ", 0, FONTHEIGTH * 4);
	displaySensorValue(rightDistance, "right ", 0, FONTHEIGTH * 5);
	displaySensorValue(vBat, "vBat ", 0, FONTHEIGTH * 6);
}
